from tko.game.quest import Quest
from tko.util.get_md_link import get_md_link
from pathlib import Path

"""
## Quest Title [@key] [+skill[:value]]... [!@quest_required]... [=lang] [=lang] [%min_percent_complete]
Values can be hided with <!-- comment -->
"""

class QuestParser:
    def __init__(self, source_alias: str):
        self.source_alias = source_alias
        self.quest = Quest().set_remote_name(source_alias)
        self.line: str = ""
        self.line_num = 0
        self.filename: Path = Path("")

    def finish_quest(self) -> Quest:
        if self.quest.get_key() == "":
            self.quest.set_key(get_md_link(self.quest.get_title()))
        return self.quest

    def match_full_pattern(self) -> bool:
        if self.line.startswith("## "):
            line = self.line[3:]
        elif self.line.startswith("### "):
            line = self.line[4:]
        else:
            return False

        line = line.replace("<!--", " ").replace("-->", " ").replace("`", " ")
        title = self.process_words(line)
        self.quest.set_title(title)
        return True

    def process_words(self, line: str) -> str:
        words = [tag.strip() for tag in line.split(" ") if tag != ""]

        keys = [tag for tag in words if tag[0] == "@"]
        if keys:
            self.quest.set_key(keys[0])

        # skills
        skills = [t[1:] for t in words if t[0] == "+"]
        if len(skills) > 0:
            self.quest.skills = {}
            for s in skills:
                try:
                    k, v = s.split(":")
                    self.quest.skills[k] = int(v)
                except ValueError:
                    self.quest.skills[s] = 1  # default value is 1 if not specified

        # languages
        languages = [t[1:] for t in words if t[0] == "="]
        if len(languages) > 0:
            self.quest.languages = []
            for l in languages:
                self.quest.languages.append(l)

        # quest percent
        qmin = [t[1:] for t in words if t[0] == "%"]
        if len(qmin) > 0:
            try:
                self.quest.min_percent_completion = int(qmin[0])
            except ValueError:
                self.quest.min_percent_completion = 50
        
        required = [t[1:] for t in words if t[0] == "!"]
        for req_key in required:
            self.quest.add_require_key(req_key)

        words = [w for w in words if w[0] not in ["@", "%", "=", "+", "!"]]
        return " ".join(words)

    def parse_quest(self, filename: Path, line: str, line_num: int) -> None | Quest:
        self.line = line
        self.line_num = line_num
        self.filename: Path = filename
        self.quest.line = self.line
        self.quest.line_number = self.line_num
        self.quest.remote_name = ""

        if self.match_full_pattern():
            return self.finish_quest()

        return None


import argparse
import markdown
from typing import Any
from tko.feno.title import FenoTitle

def convert_lists_to_4_spaces(markdown_content: str) -> str:
    """
    Converte listas no conteúdo Markdown para usar 4 espaços como indentação.

    Args:
        markdown_content (str): O conteúdo Markdown a ser processado.

    Returns:
        str: O conteúdo Markdown com listas convertidas para 4 espaços.
    """
    lines = markdown_content.splitlines()
    output_lines: list[str] = []
    for line in lines:
        lstrip = line.lstrip()
        if lstrip.startswith('- '):
            dif = len(line) - len(lstrip)
            line = ' ' * (2 * dif) + lstrip
        output_lines.append(line)
    return '\n'.join(output_lines)

def convert_markdown_to_html(title: str, markdown_file_path: str, output_html_path: str):
    try:
        with open(markdown_file_path, 'r', encoding='utf-8') as f:
            markdown_content = f.read()
        markdown_content = convert_lists_to_4_spaces(markdown_content)

        # Configura as extensões do Markdown
        # 'tables': para renderizar tabelas
        # 'fenced_code': para blocos de código com destaque de sintaxe
        # 'pymdownx.arithmatex': para renderizar LaTeX (necessita MathJax ou KaTeX no HTML)
        # 'pymdownx.highlight': para o destaque de sintaxe (depende de pygments)
        extensions = [
            'tables',
            "pymdownx.superfences",
            'pymdownx.arithmatex', # Usando arithmatex para LaTeX
            'pymdownx.highlight',  # Usando highlight para destaque de sintaxe
            'attr_list',           # Útil para atributos como classes em elementos
        ]
        extension_configs: dict[str, Any] = {
            'pymdownx.arithmatex': {
                'generic': True  # Habilita delimitadores genéricos para LaTeX ($...$ e $$...$$)
            },
            'pymdownx.highlight': {
                'css_class': 'highlight', # Classe CSS para os blocos de código
                'linenums': False,       # Não mostrar números de linha por padrão
                'pygments_lang_class': True, # Adiciona classes de linguagem para Pygments
            }
        }

        html_content = markdown.markdown(
            markdown_content,
            extensions=extensions,
            extension_configs=extension_configs
        )

        # Adiciona um CSS básico e o script do MathJax para o LaTeX
        final_html = f"""<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <style>
        body {{ font-family: sans-serif; line-height: 1.6; margin: 20px; }}
        pre {{ background-color: #f4f4f4; padding: 10px; border-radius: 5px; overflow-x: auto; }}
        code {{ font-family: monospace; }}
        table {{ border-collapse: collapse; width: 100%; }}
        th, td {{ border: 1px solid #ddd; padding: 8px; text-align: left; }}
        th {{ background-color: #f2f2f2; }}
        /* Estilos básicos para o destaque de sintaxe (gerado pelo pygments via pymdownx.highlight) */
        .highlight .hll {{ background-color: #ffffcc }}
        .highlight .c {{ color: #999988; font-style: italic }} /* Comment */
        .highlight .err {{ color: #a61717; background-color: #e3d2d2 }} /* Error */
        .highlight .k {{ color: #000000; font-weight: bold }} /* Keyword */
        .highlight .o {{ color: #000000; font-weight: bold }} /* Operator */
        .highlight .ch {{ color: #999988; font-style: italic }} /* Comment.Hashbang */
        .highlight .cm {{ color: #999988; font-style: italic }} /* Comment.Multiline */
        .highlight .cp {{ color: #999999; font-weight: bold; font-style: italic }} /* Comment.Preproc */
        .highlight .cpf {{ color: #999988; font-style: italic }} /* Comment.PreprocFile */
        .highlight .c1 {{ color: #999988; font-style: italic }} /* Comment.Single */
        .highlight .cs {{ color: #999999; font-weight: bold; font-style: italic }} /* Comment.Special */
        .highlight .gd {{ color: #000000; background-color: #ffdddd }} /* Generic.Deleted */
        .highlight .ge {{ color: #000000; font-style: italic }} /* Generic.Emph */
        .highlight .gr {{ color: #aa0000 }} /* Generic.Error */
        .highlight .gh {{ color: #999999 }} /* Generic.Heading */
        .highlight .gi {{ color: #000000; background-color: #ddffdd }} /* Generic.Inserted */
        .highlight .go {{ color: #888888 }} /* Generic.Output */
        .highlight .gp {{ color: #555555 }} /* Generic.Prompt */
        .highlight .gs {{ font-weight: bold }} /* Generic.Strong */
        .highlight .gu {{ color: #aaaaaa }} /* Generic.Subheading */
        .highlight .gt {{ color: #aa0000 }} /* Generic.Traceback */
        .highlight .kc {{ color: #000000; font-weight: bold }} /* Keyword.Constant */
        .highlight .kd {{ color: #000000; font-weight: bold }} /* Keyword.Declaration */
        .highlight .kn {{ color: #000000; font-weight: bold }} /* Keyword.Namespace */
        .highlight .kp {{ color: #000000; font-weight: bold }} /* Keyword.Pseudo */
        .highlight .kr {{ color: #000000; font-weight: bold }} /* Keyword.Reserved */
        .highlight .kt {{ color: #445588; font-weight: bold }} /* Keyword.Type */
        .highlight .m {{ color: #009999 }} /* Literal.Number */
        .highlight .s {{ color: #dd1144 }} /* Literal.String */
        .highlight .na {{ color: #008080 }} /* Name.Attribute */
        .highlight .nb {{ color: #0086B3 }} /* Name.Builtin */
        .highlight .nc {{ color: #445588; font-weight: bold }} /* Name.Class */
        .highlight .no {{ color: #008080 }} /* Name.Constant */
        .highlight .nd {{ color: #3c5d5d; font-weight: bold }} /* Name.Decorator */
        .highlight .ni {{ color: #800080 }} /* Name.Entity */
        .highlight .ne {{ color: #990000; font-weight: bold }} /* Name.Exception */
        .highlight .nf {{ color: #990000; font-weight: bold }} /* Name.Function */
        .highlight .nl {{ color: #990000; font-weight: bold }} /* Name.Label */
        .highlight .nn {{ color: #555555; font-weight: bold }} /* Name.Namespace */
        .highlight .nx {{ color: #990000; font-weight: bold }} /* Name.Other */
        .highlight .py {{ color: #009999 }} /* Name.Property */
        .highlight .p {{ color: #000000 }} /* Name.Punctuation */
        .highlight .nv {{ color: #008080 }} /* Name.Variable */
        .highlight .ow {{ color: #000000; font-weight: bold }} /* Operator.Word */
        .highlight .w {{ color: #bbbbbb }} /* Text.Whitespace */
        .highlight .mf {{ color: #009999 }} /* Literal.Number.Float */
        .highlight .mh {{ color: #009999 }} /* Literal.Number.Hex */
        .highlight .mi {{ color: #009999 }} /* Literal.Number.Integer */
        .highlight .mo {{ color: #009999 }} /* Literal.Number.Oct */
        .highlight .sb {{ color: #dd1144 }} /* Literal.String.Backtick */
        .highlight .sc {{ color: #dd1144 }} /* Literal.String.Char */
        .highlight .sd {{ color: #dd1144 }} /* Literal.String.Doc */
        .highlight .s2 {{ color: #dd1144 }} /* Literal.String.Double */
        .highlight .se {{ color: #dd1144 }} /* Literal.String.Escape */
        .highlight .sh {{ color: #dd1144 }} /* Literal.String.Heredoc */
        .highlight .si {{ color: #dd1144 }} /* Literal.String.Interpol */
        .highlight .sx {{ color: #dd1144 }} /* Literal.String.Other */
        .highlight .sr {{ color: #009926 }} /* Literal.String.Regex */
        .highlight .s1 {{ color: #dd1144 }} /* Literal.String.Single */
        .highlight .ss {{ color: #dd1144 }} /* Literal.String.Symbol */
        .highlight .bp {{ color: #0086B3 }} /* Name.Builtin.Pseudo */
        .highlight .vc {{ color: #008080 }} /* Name.Variable.Class */
        .highlight .vg {{ color: #008080 }} /* Name.Variable.Global */
        .highlight .vi {{ color: #008080 }} /* Name.Variable.Instance */
        .highlight .il {{ color: #009999 }} /* Literal.Number.Integer.Long */
    </style>
    <script type="text/javascript" async
      src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.7/MathJax.js?config=TeX-MML-AM_CHTML">
    </script>
</head>
<body>
{html_content}
</body>
</html>
"""

        with open(output_html_path, 'w', encoding='utf-8') as f:
            f.write(final_html)

        # print(f"Conversão concluída: '{markdown_file_path}' -> '{output_html_path}'")

    except FileNotFoundError:
        print(f"Erro: Arquivo Markdown não encontrado em '{markdown_file_path}'")
    except Exception as e:
        print(f"Ocorreu um erro durante a conversão: {e}")

def html_main(args: argparse.Namespace):
    # Verifica se os arquivos têm as extensões corretas
    if not args.input.endswith('.md'):
        print("Erro: O arquivo de entrada Markdown deve ter a extensão .md")
        exit(1)
    if not args.output.endswith('.html'):
        print("Erro: O arquivo de saída HTML deve ter a extensão .html")
        exit(1)

    title: str = ""
    if args.title:
        title = args.title
    else:
        title = FenoTitle.extract_title(args.input)
        
    convert_markdown_to_html(title, args.input, args.output)



from tko.util.text import Text

class TreeItem():

    def __init__(self):
        self.__remote_name: str = ""
        self.__key: str = ""
        self._title: str = ""
        self.__sentence: Text = Text()

    def get_remote_name(self) -> str:
        return self.__remote_name

    def get_full_key(self) -> str:
        return self.__remote_name + "@" + self.__key
    
    def get_key(self) -> str:
        return self.__key
    
    def get_title(self) -> str:
        return self._title
    
    def get_sentence(self) -> Text:
        return self.__sentence

    def set_remote_name(self, remote_name: str):
        self.__remote_name = remote_name
        return self
    
    def set_key(self, key: str):
        if key.startswith("@"):
            key = key[1:]
        self.__key = key
        return self

    def set_title(self, title: str):
        self._title = title
        return self
    
    def set_sentence(self, sentence: Text):
        self.__sentence = sentence
        return self



from typing import Callable
import os
import shutil

from tko.down.drafts import Drafts
from tko.settings.repository import Repository
from tko.settings.settings import Settings
from tko.game.game import Game
from tko.util.decoder import Decoder
from tko.down.drafts import Drafts
from tko.feno.remote_md import Absolute
from tko.game.task import Task
from tko.feno.filter import CodeFilter
from pathlib import Path
from tko.settings.user_languages import LangSettings

class CmdLineDown:
    def __init__(self, settings: Settings, rep: Repository, task_key: str, game: Game | None = None):
        self.settings = settings
        self.rep = rep
        self.task_key = task_key
        if game is None:
            self.rep.load_config().load_game()
            self.game = self.rep.game
        else:
            self.game = game
        
    def execute(self):
        if not self.rep.paths.has_local_config_file():
            print("O parâmetro para o comando tko down deve a pasta onde você iniciou o repositório.")
            print("Navegue ou passe o caminho até a pasta do repositório e tente novamente.")
            return False
    
        CmdDown(self.rep, self.task_key, self.settings).execute()
        return True


class CmdDown:
    test_case_filename = "tests.toml"
    def __init__(self, repo: Repository, task_key: str, settings: Settings):
        self.repo = repo
        self.task_key = task_key
        self.settings = settings
        self.task: Task = self.repo.game.get_task(self.task_key)
       
        origin_folder = self.task.get_origin_folder()
        if origin_folder is None:
            raise ValueError(f"Atividade {self.task_key} não possui pasta de origem para download")
        self.origin_folder: Path = origin_folder
        
        destiny_folder = self.task.get_workspace_folder()
        if destiny_folder is None:
            raise ValueError(f"Atividade {self.task_key} não possui pasta de destino para download")
        self.destiny_folder: Path = destiny_folder
       
        self.language: str = ""
        self.check_and_get_language()
        self.actions = DownActions(self.settings)
        
    def execute(self) -> bool:
        if self.task.is_import_type():
            return self.download_from_external_source()
        if self.task.is_static_type():
            self.actions.send_to_print("Atividade já está no repositório, precisa baixar nenhum arquivo")
            return False
        self.actions.send_to_print("falha: link para atividade não possui link para download")
        return False

    def set_fnprint(self, fnprint: Callable[[str], None]):
        self.actions.fnprint = fnprint
        return self

    def remove_empty_destiny_folder(self):
        if os.path.exists(self.destiny_folder):
            if len(os.listdir(self.destiny_folder)) == 0:
                os.rmdir(self.destiny_folder)

    def find_folder_for_drafts(self) -> Path:
        lang = self.language
        destiny_folder = self.destiny_folder
        drafts_folder = os.path.join(destiny_folder, lang)
        files_under_destiny = os.listdir(destiny_folder)
        on_root = False
        for file in files_under_destiny:
            if file.endswith(f".{lang}"):
                drafts_folder = os.path.join(destiny_folder)
                on_root = True
                break

        if not on_root and not os.path.exists(drafts_folder):
            os.makedirs(drafts_folder, exist_ok=True)
            return Path(drafts_folder)
        
        count = 1
        while True:
            drafts_backup_folder = os.path.join(destiny_folder, f"_{lang}.{count}")
            if not os.path.exists(drafts_backup_folder):
                drafts_folder = drafts_backup_folder
                os.makedirs(drafts_folder, exist_ok=True)
                break
            count += 1
        self.actions.send_to_print("")
        self.actions.send_to_print(f"Criando nova pasta de rascunhos: {os.path.basename(drafts_folder)} ")
        self.actions.send_to_print(f"")
        self.actions.send_to_print(f"Se quiser utilizar os novos rascunhos, copie manualmente ")
        self.actions.send_to_print(f"os novos rascunhos para a pasta {lang} ")
        return Path(drafts_folder)

    def remove_draft_folder_if_duplicated(self, drafts_folder: Path) -> tuple[bool, Path]:
        lang = self.language
        destiny_folder = drafts_folder.parent
        parts = drafts_folder.name.split(".")
        if len(parts) > 1:
            try:
                count = int(parts[-1])
                if count == 1:
                    last_path = os.path.join(destiny_folder, lang)
                else:
                    last = count - 1
                    last_path = ".".join(parts[:-1]) + f".{last}"
                if os.path.exists(last_path):
                    if self.folder_equals(drafts_folder, Path(last_path)):
                        shutil.rmtree(drafts_folder)
                        return True, Path(last_path)
            except ValueError:
                pass
        return False, Path("")

    def download_from_external_source(self) -> bool:
        os.makedirs(self.destiny_folder, exist_ok=True)

        self.copy_readme()
        self.copy_tests()

        self.actions.cached = True
        destiny_drafts_folder = self.find_folder_for_drafts()
        origin_source = os.path.join(CodeFilter.get_default_drafts_dir(self.origin_folder), self.language)
        if not self.copy_drafts_from(Path(origin_source), Path(destiny_drafts_folder)):
            self.actions.create_default_draft(destiny_drafts_folder, self.language)
        if self.task.task_rate == self.task.TaskRate.USER:
            self.actions.create_default_draft(destiny_drafts_folder, "md")

        removed, last_path = self.remove_draft_folder_if_duplicated(destiny_drafts_folder)
        self.actions.cached = False
        if not removed:
            for msg in self.actions.cache_msgs:
                self.actions.send_to_print(msg)
            self.actions.cache_msgs.clear()
        else:
            self.actions.send_to_print(f"Último rascunho em {os.path.basename(last_path)}")
        self.actions.send_to_print("")
        self.actions.send_to_print("Atividade baixada com sucesso")
                
        return True
    
    def copy_readme(self):
        origin_readme = os.path.join(self.origin_folder, "README.md")
        destiny_readme = os.path.join(self.destiny_folder, "README.md")

        source_folder_rel = os.path.relpath(self.origin_folder, self.destiny_folder)
        content = Decoder.load(origin_readme)
        content = Absolute.change_to_relative_folder(content, source_folder_rel)
        self.actions.compare_and_save_to(content, destiny_readme)
    
    def copy_drafts_from(self, origin_drafts_folder: Path, destiny_draft_folder: Path) -> bool:
        if not os.path.exists(origin_drafts_folder):
            return False
        os.makedirs(destiny_draft_folder, exist_ok=True)
        if self.copy_drafts_from_cache(origin_drafts_folder, destiny_draft_folder):
            return True
        return False
        

    def copy_drafts_from_cache(self, cache_draft_folder: Path, destiny_draft_folder: Path) -> bool:
        found: bool = False
        for file in os.listdir(cache_draft_folder):
            source_path = os.path.join(cache_draft_folder, file)
            destiny_path = os.path.join(destiny_draft_folder, file)
            if os.path.isfile(source_path):
                self.actions.compare_and_save_to(Decoder.load(source_path), destiny_path)
                found = True
        return found
        

    def copy_tests(self):
        source_folder = self.origin_folder
        destiny_folder = self.destiny_folder
        # copy any file with extension .toml or .tio from source_folder to destiny_folder, if they are different
        for file in os.listdir(source_folder):
            if file.endswith(".toml") or file.endswith(".tio"):
                source_path = os.path.join(source_folder, file)
                destiny_path = os.path.join(destiny_folder, file)
                if os.path.isfile(source_path):
                    self.actions.compare_and_save_to(Decoder.load(source_path), destiny_path)


    @staticmethod
    def folder_equals(folder1: Path, folder2: Path) -> bool:
        """ Compare two folders and return if they have the same files with the same content """
        if not folder1.exists() or not folder2.exists():
            return False
        if len(os.listdir(folder1)) != len(os.listdir(folder2)):
            return False
        for file in os.listdir(folder1):
            path1 = os.path.join(folder1, file)
            path2 = os.path.join(folder2, file)
            if not os.path.isfile(path1) or not os.path.isfile(path2):
                return False
            with open(path1, "rb") as f1, open(path2, "rb") as f2:
                if f1.read() != f2.read():
                    return False
        return True

    def check_and_get_language(self) -> None:
        language_def = self.repo.data.get_lang()

        if self.language == "":
            if language_def != "":
                self.language = language_def
            else:
                print("Escolha uma extensão para os rascunhos: [{}]: ".format(", ".join(Drafts().get_languages_with_drafts())), end="")
                self.language = input()

class DownActions:
    def __init__(self, settings: Settings):
        self.settings = settings
        self.fnprint: Callable[[str], None] = print
        self.cache_msgs: list[str] = []
        self.cached: bool = False

    def send_to_print(self, text: str):
        if self.cached:
            self.cache_msgs.append(text)
        else:
            self.fnprint(text)

    @staticmethod
    def folder_and_file(path: str, parts: int = 2) -> str:
        """ Return the folder and file name of the path """
        pieces = path.split(os.sep)
        pieces = pieces[-parts:]  # Get the last 'parts' elements
        return os.path.join(*pieces)

    def compare_and_save_to(self, content: str, path: str):
        if not os.path.exists(path):
            Decoder.save(path, content)
            self.send_to_print(self.folder_and_file(path) + " (Novo)")
        else:
            path_content = Decoder.load(path)
            if path_content != content:
                self.send_to_print(self.folder_and_file(path) + " (Atualizado)")
                Decoder.save(path, content)
            else:
                self.send_to_print(self.folder_and_file(path) + " (Inalterado)")

    def create_default_draft(self, destiny: Path, language: str) -> str:
        filename = "draft."
        draft_path = os.path.join(destiny, filename + language)
        os.makedirs(os.path.dirname(draft_path), exist_ok=True)
        languages: dict[str, LangSettings] = self.settings.languages.lang_settings
        if not os.path.exists(draft_path):
            with open(draft_path, "w", encoding="utf-8") as f:
                if language in Drafts.drafts:
                    f.write(Drafts.drafts[language])
                elif language in languages.keys():
                    f.write(languages[language].draft)  
                else:
                    f.write("")
            self.send_to_print(self.folder_and_file(draft_path, 3) + " (Vazio)")
        else:
            self.send_to_print(self.folder_and_file(draft_path, 3) + " (Não sobrescrito)")
        return draft_path



from tko.cmds.cmd_run import Run
from tko.util.param import Param
from tko.settings.settings import Settings
from tko.enums.diff_count import DiffCount
from tko.util.text import Text
from tko.util.symbols import symbols
from pathlib import Path

class CmdEval:
    EVAL_TIMEOUT_DEFAULT = 30
    
    def __init__(self):
        self.target_list: list[str] = []
        self.no_run: bool = False
        self.track: bool = False
        self.load_self: bool = False
        self.complex: bool = False
        self.timeout: int = CmdEval.EVAL_TIMEOUT_DEFAULT
        self.result_file: str | None = None
        self.diff = DiffCount.FIRST

        symbols.execution_result["untested"] = Text.Token("U", "")
        symbols.execution_result["success"] = Text.Token("S", "g")
        symbols.execution_result["wrong_output"] = Text.Token("W", "r")
        symbols.execution_result["compilation_error"] = Text.Token("C", "m")
        symbols.execution_result["execution_error"] = Text.Token("E", "y")

    def set_diff(self, diff: DiffCount):
        self.diff = diff
        return self

    def set_no_run(self, value: bool | None = None):
        if value is not None:
            self.no_run = value
        return self
    
    def set_track(self, value: bool | None = None):
        if value is not None:
            self.track = value
        return self
    
    def set_self(self, value: bool | None = None):
        if value is not None:
            self.load_self = value
        return self
    
    def set_complex(self, value: bool | None = None):
        if value is not None:
            self.complex = value
        return self
    
    def set_timeout(self, timeout: int | None = None):
        if timeout is not None:
            self.timeout = timeout
        else:
            self.timeout = CmdEval.EVAL_TIMEOUT_DEFAULT
        return self

    def set_result_file(self, result_file: str | None = None):
        self.result_file = result_file
        return self
    
    def set_target_list(self, target_list: list[str]):
        self.target_list = target_list
        return self

    def execute(self) -> None:
        if self.no_run and not self.load_self:
            print("Nothing to do. If using --norun, you should choice at least --self.")
            return
        
        param = Param.Basic()
        param.set_diff_count(self.diff)
        param.set_compact(True)

        settings = Settings()
        cmd_run = Run(settings, [Path(x) for x in self.target_list], param)
        cmd_run.set_eval_mode().set_abort_on_exec_error()
        if self.no_run:
            cmd_run.set_no_run()
        if self.track:
            cmd_run.show_track_info()
        if self.load_self:
            cmd_run.show_self_info()
        if self.complex:
            cmd_run.set_complex_percent()
        if self.timeout != 0:
            cmd_run.set_timeout(self.timeout)
        percent = cmd_run.execute()

        if self.result_file:
            with open(self.result_file, 'w') as f:
                f.write(f"{percent}%\n")


from urllib.request import urlopen
from datetime import datetime as dt
from tko.settings.settings import Settings
from tko import __version__

class CheckVersion:
    link = "https://raw.githubusercontent.com/senapk/tko/master/src/tko/__init__.py"

    def __init__(self, settings: Settings):
        self.settings: Settings = settings
        self.user_version: str = __version__
        
    def is_updated(self):
        latest_version = self.get_latest_version()
        if latest_version == "":
            return True
        if self.user_version != latest_version:
            major, minor, patch = [int(x) for x in self.user_version.split(".")]
            latest_major, latest_minor, latest_patch = [int(x) for x in latest_version.split(".")]
            if major < latest_major or (major == latest_major and minor < latest_minor) or (major == latest_major and minor == latest_minor and patch < latest_patch):
                return False
        return True

    def get_latest_version(self) -> str:
        now = dt.now()
        last_check_str = self.settings.app.get_lastest_version_check_timestamp()
        stored_latest = self.settings.app.get_lastest_version()
        if last_check_str != "" and stored_latest != "":
            last_check = dt.fromisoformat(last_check_str)
            if (now - last_check).total_seconds() < 60 * 60 * 24: # 24h
                return stored_latest
        try:
            with urlopen(self.link, timeout=1) as f:
                for line in f:
                    if b"__version__" in line:
                        latest = line.decode().split('"')[1]
                        self.settings.app.set_latest_version_check_timestamp(now.isoformat())
                        self.settings.app.set_lastest_version(latest)
                        self.settings.save_settings()
                        return latest
        except:
            pass
        return ""


from pathlib import Path

class Decoder:

    @staticmethod


    def load(file_path: str | Path) -> str:
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                return f.read().replace("\r\n", "\n")
        except UnicodeDecodeError:
            with open(file_path, "r", encoding="latin-1") as f:
                return f.read().replace("\r\n", "\n")

    @staticmethod
    def save(file_path: str | Path, content: str):
        file_path = Path(file_path)
        base_path = file_path.parent
        base_path.mkdir(parents=True, exist_ok=True)
        with open(file_path, "w", encoding="utf-8", newline='\n') as file:
            file.write(content)
